@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
  *
  * Authors:
@@ -31,14 +31,10 @@
 #include <errno.h>
 
 #include <glib.h>
+#include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 #include <gio/gio.h>
 
-#include <camel/camel-private.h>
-#include <glib/gi18n-lib.h>
-#include <camel/camel-net-utils.h>
-#include <camel/camel-debug.h>
-
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
 #include "camel-mapi-store-summary.h"
@@ -53,7 +49,7 @@ extern gint camel_application_is_exiting;
 
 struct mapi_push_notification_data {
 	guint16 event_mask;
-	guint32 connection; 
+	guint32 connection;
 	guint32 event_options;
 	gpointer event_data;
 
@@ -69,6 +65,7 @@ process_mapi_new_mail_notif (CamelMapiStore *store, struct NewMailNotification *
 
 	fetch_items_data *fetch_data;
 	CamelFolder *folder = NULL;
+	CamelStore *parent_store;
 	gint info_count = -1;
 	CamelStoreInfo *info;
 	CamelMapiStoreInfo *mapi_info;
@@ -90,7 +87,7 @@ process_mapi_new_mail_notif (CamelMapiStore *store, struct NewMailNotification *
 	while (info_count >= 0) {
 		info = camel_store_summary_index ((CamelStoreSummary *)store->summary, info_count);
 		mapi_info = (CamelMapiStoreInfo *)info;
-		if (info && !g_strcmp0 (mapi_info->folder_id, folder_id)){
+		if (info && !g_strcmp0 (mapi_info->folder_id, folder_id)) {
 			folder_name = mapi_info->full_name;
 		}
 		if (info)
@@ -104,6 +101,8 @@ process_mapi_new_mail_notif (CamelMapiStore *store, struct NewMailNotification *
 	if (!folder)
 		return;
 
+	parent_store = camel_folder_get_parent_store (folder);
+
 	/*Use restriction to fetch the message summary based on MID*/
 	res = g_new0 (struct mapi_SRestriction, 1);
 
@@ -117,15 +116,15 @@ process_mapi_new_mail_notif (CamelMapiStore *store, struct NewMailNotification *
 	fetch_data->changes = camel_folder_change_info_new ();
 	fetch_data->folder = folder;
 
-	CAMEL_SERVICE_REC_LOCK (store, connect_lock);
+	camel_service_lock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	camel_mapi_folder_fetch_summary ((CamelStore *)store, new_mail_notif->FID, res, NULL, fetch_data, options);
-	CAMEL_SERVICE_REC_UNLOCK (store, connect_lock);
-	
+	camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+
 	camel_folder_summary_touch (folder->summary);
 	/* mapi_sync_summary */
 	camel_folder_summary_save_to_db (folder->summary, NULL);
-	camel_store_summary_touch ((CamelStoreSummary *)((CamelMapiStore *)folder->parent_store)->summary);
-	camel_store_summary_save ((CamelStoreSummary *)((CamelMapiStore *)folder->parent_store)->summary);
+	camel_store_summary_touch ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary);
+	camel_store_summary_save ((CamelStoreSummary *)((CamelMapiStore *)parent_store)->summary);
 
 	camel_object_trigger_event (folder, "folder_changed", fetch_data->changes);
 
@@ -134,9 +133,9 @@ process_mapi_new_mail_notif (CamelMapiStore *store, struct NewMailNotification *
 }
 
 static gint
-mapi_notifications_filter (guint16 type, void *event, void *store)
+mapi_notifications_filter (guint16 type, gpointer event, gpointer store)
 {
-	switch(type) {
+	switch (type) {
 	/* -- Folder Events -- */
 	case fnevObjectCreated:
 		d_notifications(printf ("Event : Folder Created\n"));
@@ -185,7 +184,6 @@ mapi_notifications_filter (guint16 type, void *event, void *store)
 	return 0;
 }
 
-
 /*Of type mapi_notify_continue_callback_t*/
 static gint
 mapi_notifications_continue_check (gpointer data)
@@ -197,7 +195,7 @@ mapi_notifications_continue_check (gpointer data)
 	if (g_cancellable_is_cancelled (thread_data->cancellable) || (camel_application_is_exiting == TRUE))
 		return 1;
 
-	/* HACK ALERT : This is a BAD idea. But ;-), A bug in MonitorNotification */
+	/* HACK ALERT : This is a BAD idea. But;-), A bug in MonitorNotification */
 	/* makes select() return immediately. We are introducing a artificial delay here */
 	/* to avoid high CPU usage. Remove this when libmapi 0.9.1 is out */
 	g_usleep (G_USEC_PER_SEC * 2);
@@ -211,6 +209,7 @@ mapi_push_notification_listener_thread (gpointer data)
 	struct mapi_push_notification_data *thread_data = data;
 	CamelMapiStore *mapi_store = (CamelMapiStore *) thread_data->event_data;
 	struct mapi_notify_continue_callback_data *cb_data = g_new0 (struct mapi_notify_continue_callback_data, 1);
+	ExchangeMapiConnection *conn;
 
 	g_return_val_if_fail (data != NULL, NULL);
 
@@ -222,20 +221,29 @@ mapi_push_notification_listener_thread (gpointer data)
 	cb_data->callback = mapi_notifications_continue_check;
 	cb_data->data = thread_data;
 
-	CAMEL_SERVICE_REC_LOCK (mapi_store, connect_lock);
+	camel_service_lock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
-	if (exchange_mapi_events_init ()) {
-		exchange_mapi_events_subscribe (thread_data->event_options, thread_data->event_mask,
+	conn = camel_mapi_store_get_exchange_connection (mapi_store);
+	if (!conn) {
+		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		g_return_val_if_reached (NULL);
+	}
+
+	g_object_ref (conn);
+
+	if (exchange_mapi_connection_events_init (conn)) {
+		exchange_mapi_connection_events_subscribe (conn, thread_data->event_options, thread_data->event_mask,
 						&thread_data->connection, mapi_notifications_filter,
 						thread_data->event_data);
 
-		CAMEL_SERVICE_REC_UNLOCK (mapi_store, connect_lock);
-		exchange_mapi_events_monitor (cb_data); /*Blocking call. Don't hold locks here*/
-		exchange_mapi_events_unsubscribe (thread_data->connection);
-	} else 
-		CAMEL_SERVICE_REC_UNLOCK (mapi_store, connect_lock);
+		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
+		exchange_mapi_connection_events_monitor (conn, cb_data); /*Blocking call. Don't hold locks here*/
+		exchange_mapi_connection_events_unsubscribe (conn, thread_data->connection);
+	} else
+		camel_service_unlock (CAMEL_SERVICE (mapi_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	g_free (cb_data);
+	g_object_unref (conn);
 
 	return NULL;
 }
@@ -279,9 +287,9 @@ camel_mapi_notification_listener_stop (CamelMapiStore *mstore, gpointer start_va
 	g_object_unref (thread_data->cancellable);
 	g_free (thread_data);
 
-	camel_object_ref (mstore);
-	CAMEL_SERVICE_REC_LOCK (mstore, connect_lock);
+	g_object_ref (mstore);
+	camel_service_lock (CAMEL_SERVICE (mstore), CAMEL_SERVICE_REC_CONNECT_LOCK);
 	camel_mapi_store_unset_notification_data (mstore);
-	CAMEL_SERVICE_REC_UNLOCK (mstore, connect_lock);
-	camel_object_unref (mstore);
+	camel_service_unlock (CAMEL_SERVICE (mstore), CAMEL_SERVICE_REC_CONNECT_LOCK);
+	g_object_unref (mstore);
 }
