@@ -11,7 +11,7 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with the program; if not, see <http://www.gnu.org/licenses/>  
+ * License along with the program; if not, see <http://www.gnu.org/licenses/>
  *
  *
  * Authors:
@@ -21,38 +21,23 @@
  *
  */
 
-
 /* -- Generate MIME to ITEM -- */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
 
+#include <glib/gi18n-lib.h>
+
 #include <libmapi/libmapi.h>
 #include <gen_ndr/exchange.h>
 
-#include <camel/camel-data-wrapper.h>
-#include <camel/camel-exception.h>
-#include <camel/camel-mime-filter-crlf.h>
-#include <camel/camel-mime-message.h>
-#include <camel/camel-multipart.h>
-#include <camel/camel-session.h>
-#include <camel/camel-stream-filter.h>
-#include <camel/camel-stream-mem.h>
-
-#include <camel/camel-private.h>
-#include <glib/gi18n-lib.h>
-#include <camel/camel-net-utils.h>
-#include <camel/camel-seekable-stream.h>
-#include <camel/camel-sasl.h>
-#include <camel/camel-utf8.h>
-#include <camel/camel-tcp-stream-raw.h>
-
-#ifdef HAVE_SSL
-#include <camel/camel-tcp-stream-ssl.h>
-#endif
-
 #include <exchange-mapi-defs.h>
+#include "exchange-mapi-utils.h"
 
 #include "camel-mapi-store.h"
 #include "camel-mapi-folder.h"
@@ -64,15 +49,15 @@
 #define STREAM_SIZE 4000
 
 static void
-mapi_item_add_recipient (const char *recipients, OlMailRecipientType type, GSList **recipient_list)
+mapi_item_add_recipient (const gchar *recipients, OlMailRecipientType type, GSList **recipient_list)
 {
 	ExchangeMAPIRecipient *recipient;
 	uint32_t val = 0;
 	uint8_t bVal;
-	const char *str = NULL;
+	const gchar *str = NULL;
 
 	if (!recipients)
-		return ;
+		return;
 
 	recipient = g_new0 (ExchangeMAPIRecipient, 1);
 
@@ -83,21 +68,21 @@ mapi_item_add_recipient (const char *recipients, OlMailRecipientType type, GSLis
 	recipient->in.req_cValues = 2;
 	recipient->in.req_lpProps = g_new0 (struct SPropValue, recipient->in.req_cValues + 1);
 
-	set_SPropValue_proptag (&(recipient->in.req_lpProps[0]), PR_RECIPIENT_TYPE, (const void *) &type);
+	set_SPropValue_proptag (&(recipient->in.req_lpProps[0]), PR_RECIPIENT_TYPE, (gconstpointer ) &type);
 
 	val = 0;
-	set_SPropValue_proptag (&(recipient->in.req_lpProps[1]), PR_SEND_INTERNET_ENCODING, (const void *)&val);
+	set_SPropValue_proptag (&(recipient->in.req_lpProps[1]), PR_SEND_INTERNET_ENCODING, (gconstpointer )&val);
 
 	/* External recipient properties - set them only when the recipient is unresolved */
 	recipient->in.ext_cValues = 8;
 	recipient->in.ext_lpProps = g_new0 (struct SPropValue, recipient->in.ext_cValues + 1);
 
 	val = DT_MAILUSER;
-	set_SPropValue_proptag (&(recipient->in.ext_lpProps[0]), PR_DISPLAY_TYPE, (const void *)&val);
+	set_SPropValue_proptag (&(recipient->in.ext_lpProps[0]), PR_DISPLAY_TYPE, (gconstpointer )&val);
 	val = MAPI_MAILUSER;
-	set_SPropValue_proptag (&(recipient->in.ext_lpProps[1]), PR_OBJECT_TYPE, (const void *)&val);
+	set_SPropValue_proptag (&(recipient->in.ext_lpProps[1]), PR_OBJECT_TYPE, (gconstpointer )&val);
 	str = "SMTP";
-	set_SPropValue_proptag (&(recipient->in.ext_lpProps[2]), PR_ADDRTYPE_UNICODE, (const void *)(str));
+	set_SPropValue_proptag (&(recipient->in.ext_lpProps[2]), PR_ADDRTYPE_UNICODE, (gconstpointer )(str));
 	str = recipient->email_id;
 	set_SPropValue_proptag (&(recipient->in.ext_lpProps[3]), PR_SMTP_ADDRESS_UNICODE, (gconstpointer )(str));
 	/* FIXME: Please add the correct names here instead of the e-mail ID */
@@ -112,7 +97,7 @@ mapi_item_add_recipient (const char *recipients, OlMailRecipientType type, GSLis
 }
 
 static void
-mapi_item_set_from(MapiItem *item, const char *from)
+mapi_item_set_from(MapiItem *item, const gchar *from)
 {
 	if (item->header.from)
 		g_free (item->header.from);
@@ -121,7 +106,7 @@ mapi_item_set_from(MapiItem *item, const char *from)
 }
 
 static void
-mapi_item_set_subject(MapiItem *item, const char *subject)
+mapi_item_set_subject(MapiItem *item, const gchar *subject)
 {
 	if (item->header.subject)
 		g_free (item->header.subject);
@@ -143,7 +128,7 @@ mapi_item_set_body_stream (MapiItem *item, CamelStream *body, MapiItemPartType p
 
 	stream->value = g_byte_array_new ();
 
-	while (read_size = camel_stream_read (body, (char *)buf, STREAM_SIZE), read_size > 0){
+	while (read_size = camel_stream_read (body, (gchar *)buf, STREAM_SIZE), read_size > 0) {
 		stream->value = g_byte_array_append (stream->value, buf, read_size);
 
 		is_null_terminated = buf [read_size - 1] == 0;
@@ -169,12 +154,6 @@ mapi_item_set_body_stream (MapiItem *item, CamelStream *body, MapiItemPartType p
 			g_byte_array_append (stream->value, (const guint8 *)"", 1);
 
 		item->msg.body_parts = g_slist_append (item->msg.body_parts, stream);
-	} else if (stream->proptag == PR_HTML) {
-		/* PR_HTML shouldn't be in UTF-16 */
-		if (!is_null_terminated)
-			g_byte_array_append (stream->value, (const guint8 *)"", 1);
-
-		item->generic_streams = g_slist_append (item->generic_streams, stream);
 	} else {
 		gsize written = 0;
 		gchar *in_unicode;
@@ -208,71 +187,71 @@ mapi_item_add_attach (MapiItem *item, CamelMimePart *part, CamelStream *content_
 	CamelContentType *content_type;
 
 	ExchangeMAPIAttachment *item_attach;
-	ExchangeMAPIStream *stream; 
+	ExchangeMAPIStream *stream;
 
 	const gchar *filename = camel_mime_part_get_filename (part);
-	
+
 	item_attach = g_new0 (ExchangeMAPIAttachment, 1);
 
 	item_attach->lpProps = g_new0 (struct SPropValue, 6 + 1);
 
-	flag = ATTACH_BY_VALUE; 
-	set_SPropValue_proptag(&(item_attach->lpProps[i++]), PR_ATTACH_METHOD, (const void *) (&flag));
+	flag = ATTACH_BY_VALUE;
+	set_SPropValue_proptag(&(item_attach->lpProps[i++]), PR_ATTACH_METHOD, (gconstpointer ) (&flag));
 
-	/* MSDN Documentation: When the supplied offset is -1 (0xFFFFFFFF), the 
-	 * attachment is not rendered using the PR_RENDERING_POSITION property. 
-	 * All values other than -1 indicate the position within PR_BODY at which 
-	 * the attachment is to be rendered. 
+	/* MSDN Documentation: When the supplied offset is -1 (0xFFFFFFFF), the
+	 * attachment is not rendered using the PR_RENDERING_POSITION property.
+	 * All values other than -1 indicate the position within PR_BODY at which
+	 * the attachment is to be rendered.
 	 */
 	flag = 0xFFFFFFFF;
-	set_SPropValue_proptag(&(item_attach->lpProps[i++]), PR_RENDERING_POSITION, (const void *) (&flag));
+	set_SPropValue_proptag(&(item_attach->lpProps[i++]), PR_RENDERING_POSITION, (gconstpointer ) (&flag));
 
 	if (filename) {
-		set_SPropValue_proptag(&(item_attach->lpProps[i++]), 
+		set_SPropValue_proptag(&(item_attach->lpProps[i++]),
 				       PR_ATTACH_FILENAME_UNICODE,
-				       (const void *) g_strdup(filename));
+				       (gconstpointer ) g_strdup(filename));
 
-		set_SPropValue_proptag(&(item_attach->lpProps[i++]), 
+		set_SPropValue_proptag(&(item_attach->lpProps[i++]),
 				       PR_ATTACH_LONG_FILENAME_UNICODE,
-				       (const void *) g_strdup(filename));
+				       (gconstpointer ) g_strdup(filename));
 	}
 
 	/* mime type : multipart/related */
 	content_id = camel_mime_part_get_content_id (part);
 	if (content_id) {
-		set_SPropValue_proptag(&(item_attach->lpProps[i++]), 
+		set_SPropValue_proptag(&(item_attach->lpProps[i++]),
 				       PR_ATTACH_CONTENT_ID,
-				       (const void *) g_strdup(content_id));
+				       (gconstpointer ) g_strdup(content_id));
 	}
 
 	content_type  = camel_mime_part_get_content_type (part);
 	if (content_type) {
 		gchar *ct = camel_content_type_simple (content_type);
 		if (ct) {
-			set_SPropValue_proptag (&(item_attach->lpProps[i++]), 
+			set_SPropValue_proptag (&(item_attach->lpProps[i++]),
 					PR_ATTACH_MIME_TAG,
-					(const void *) ct);
+					(gconstpointer ) ct);
 		}
 	}
 
 	item_attach->cValues = i;
 
 	stream = g_new0 (ExchangeMAPIStream, 1);
-	stream->proptag = PR_ATTACH_DATA_BIN; 
+	stream->proptag = PR_ATTACH_DATA_BIN;
 	stream->value = g_byte_array_new ();
 
 	camel_seekable_stream_seek((CamelSeekableStream *)content_stream, 0, CAMEL_STREAM_SET);
-	while (read_size = camel_stream_read(content_stream, (char *)buf, STREAM_SIZE), read_size > 0) {
+	while (read_size = camel_stream_read(content_stream, (gchar *)buf, STREAM_SIZE), read_size > 0) {
 		stream->value = g_byte_array_append (stream->value, buf, read_size);
 	}
 
-	item_attach->streams = g_slist_append (item_attach->streams, stream); 
+	item_attach->streams = g_slist_append (item_attach->streams, stream);
 	item->attachments = g_slist_append(item->attachments, item_attach);
 
 	return TRUE;
 }
 
-static gboolean 
+static gboolean
 mapi_do_multipart (CamelMultipart *mp, MapiItem *item, gboolean *is_first)
 {
 	CamelDataWrapper *dw;
@@ -291,12 +270,12 @@ mapi_do_multipart (CamelMultipart *mp, MapiItem *item, gboolean *is_first)
 	for (i_part = 0; i_part < n_part; i_part++) {
 		/* getting part */
 		part = camel_multipart_get_part(mp, i_part);
-		dw = camel_medium_get_content_object (CAMEL_MEDIUM (part));
+		dw = camel_medium_get_content (CAMEL_MEDIUM (part));
 		if (CAMEL_IS_MULTIPART(dw)) {
 			/* recursive */
 			if (!mapi_do_multipart (CAMEL_MULTIPART (dw), item, is_first))
 				return FALSE;
-			continue ;
+			continue;
 		}
 		/* filename */
 		filename = camel_mime_part_get_filename(part);
@@ -308,7 +287,7 @@ mapi_do_multipart (CamelMultipart *mp, MapiItem *item, gboolean *is_first)
 
 		description = camel_mime_part_get_description(part);
 		content_id = camel_mime_part_get_content_id(part);
-		
+
 		type = camel_mime_part_get_content_type(part);
 
 		if (i_part == 0 && (*is_first) && camel_content_type_is (type, "text", "plain")) {
@@ -324,7 +303,6 @@ mapi_do_multipart (CamelMultipart *mp, MapiItem *item, gboolean *is_first)
 	return TRUE;
 }
 
-
 MapiItem *
 camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, CamelException *ex)
 {
@@ -332,20 +310,20 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, Ca
 	CamelContentType *type;
 	CamelStream *content_stream;
 	CamelMultipart *multipart;
-	const CamelInternetAddress *to, *cc, *bcc;
+	CamelInternetAddress *to, *cc, *bcc;
 	MapiItem *item = g_new0 (MapiItem, 1);
-	const char *namep;
-	const char *addressp;
-	const char *content_type;		
+	const gchar *namep;
+	const gchar *addressp;
+	const gchar *content_type;
 
-	ssize_t	content_size;
+	gssize	content_size;
 	GSList *recipient_list = NULL;
 	gint i = 0;
 
 	/* headers */
 
 	if (from) {
-		if (!camel_internet_address_get ((const CamelInternetAddress *)from, 0, &namep, &addressp)) {
+		if (!camel_internet_address_get ((CamelInternetAddress *)from, 0, &namep, &addressp)) {
 			g_warning ("%s: Invalid 'from' passed in", G_STRFUNC);
 			g_free (item);
 			return NULL;
@@ -358,7 +336,7 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, Ca
 	mapi_item_set_from (item, namep);
 
 	to = camel_mime_message_get_recipients(message, CAMEL_RECIPIENT_TYPE_TO);
-	for (i = 0; to && camel_internet_address_get (to, i, &namep, &addressp); i++){
+	for (i = 0; to && camel_internet_address_get (to, i, &namep, &addressp); i++) {
 		mapi_item_add_recipient (addressp, olTo, &recipient_list);
 	}
 
@@ -371,7 +349,7 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, Ca
 	for (i = 0; bcc && camel_internet_address_get (bcc, i, &namep, &addressp); i++) {
 		mapi_item_add_recipient (addressp, olBCC, &recipient_list);
 	}
-	
+
 	if (camel_mime_message_get_subject(message)) {
 		mapi_item_set_subject(item, camel_mime_message_get_subject(message));
 	}
@@ -382,14 +360,14 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, Ca
 	item->header.message_id = g_strdup (camel_medium_get_header ((CamelMedium *) message, "Message-ID"));
 
 	/* contents body */
-	multipart = (CamelMultipart *)camel_medium_get_content_object (CAMEL_MEDIUM (message));
+	multipart = (CamelMultipart *)camel_medium_get_content (CAMEL_MEDIUM (message));
 
 	if (CAMEL_IS_MULTIPART(multipart)) {
 		gboolean is_first = TRUE;
 		if (!mapi_do_multipart (CAMEL_MULTIPART(multipart), item, &is_first))
-			printf("camel message multi part error\n"); 
+			printf("camel message multi part error\n");
 	} else {
-		dw = camel_medium_get_content_object (CAMEL_MEDIUM (message));
+		dw = camel_medium_get_content (CAMEL_MEDIUM (message));
 		if (dw) {
 			type = camel_mime_part_get_content_type((CamelMimePart *)message);
 			content_type = camel_content_type_simple (type);
@@ -402,45 +380,46 @@ camel_mapi_utils_mime_to_item (CamelMimeMessage *message, CamelAddress *from, Ca
 	}
 
 	item->recipients = recipient_list;
-	
+
 	return item;
 }
 
-gint
-camel_mapi_utils_create_item_build_props (struct SPropValue **value, struct SPropTagArray *SPropTagArray, gpointer data)
+gboolean
+camel_mapi_utils_create_item_build_props (ExchangeMapiConnection *conn, mapi_id_t fid, TALLOC_CTX *mem_ctx, struct SPropValue **values, uint32_t *n_values, gpointer data)
 {
 
 	MapiItem *item = (MapiItem *) data;
-	struct SPropValue *props;
 	GSList *l;
-	bool *send_rich_info = g_new0 (bool, 1);
-	uint32_t *msgflag = g_new0 (uint32_t, 1);
-	uint32_t *cpid = g_new0 (uint32_t, 1);
-	int i=0;
+	bool send_rich_info;
+	uint32_t msgflag;
+	uint32_t cpid;
 
-	props = g_new0 (struct SPropValue, 11 + 1);
+	#define set_value(hex, val) G_STMT_START { \
+		if (!exchange_mapi_utils_add_spropvalue (mem_ctx, values, n_values, hex, val)) \
+			return FALSE;	\
+		} G_STMT_END
 
-	*cpid = 65001; /* UTF8 - also used with PR_HTML */
-	set_SPropValue_proptag(&props[i++], PR_INTERNET_CPID, cpid);
-	set_SPropValue_proptag(&props[i++], PR_SUBJECT_UNICODE, g_strdup (item->header.subject));
-	/* PR_CONVERSATION_TOPIC_UNICODE and PR_NORMALIZED_SUBJECT_UNICODE, together with PR_SUBJECT_PREFIX_UNICODE
-	   are computed from PR_SUBJECT by the server */
+	cpid = 65001; /* UTF8 */
+	set_value (PR_INTERNET_CPID, &cpid);
+	set_value (PR_SUBJECT_UNICODE, item->header.subject);
+	set_value (PR_CONVERSATION_TOPIC_UNICODE, item->header.subject);
+	set_value (PR_NORMALIZED_SUBJECT_UNICODE, item->header.subject);
 
-	*send_rich_info = false;
-	set_SPropValue_proptag(&props[i++], PR_SEND_RICH_INFO, (const void *) send_rich_info);
+	send_rich_info = false;
+	set_value (PR_SEND_RICH_INFO, &send_rich_info);
 
-	*msgflag = MSGFLAG_UNSENT;
-	set_SPropValue_proptag(&props[i++], PR_MESSAGE_FLAGS, (void *)msgflag);
+	msgflag = MSGFLAG_UNSENT;
+	set_value (PR_MESSAGE_FLAGS, &msgflag);
 
 	/* Message threading information */
 	if (item->header.references)
-		set_SPropValue_proptag(&props[i++], PR_INTERNET_REFERENCES, g_strdup (item->header.references));
+		set_value (PR_INTERNET_REFERENCES, item->header.references);
 
 	if (item->header.in_reply_to)
-		set_SPropValue_proptag(&props[i++], PR_IN_REPLY_TO_ID, g_strdup (item->header.in_reply_to));
+		set_value (PR_IN_REPLY_TO_ID, item->header.in_reply_to);
 
 	if (item->header.message_id)
-		set_SPropValue_proptag(&props[i++], PR_INTERNET_MESSAGE_ID, g_strdup (item->header.message_id));
+		set_value (PR_INTERNET_MESSAGE_ID, item->header.message_id);
 
 	for (l = item->msg.body_parts; l; l = l->next) {
 		ExchangeMAPIStream *stream = (ExchangeMAPIStream *) (l->data);
@@ -449,17 +428,16 @@ camel_mapi_utils_create_item_build_props (struct SPropValue **value, struct SPro
 		bin->cb = stream->value->len;
 		bin->lpb = (uint8_t *)stream->value->data;
 		if (stream->proptag == PR_HTML)
-			set_SPropValue_proptag(&props[i++], stream->proptag, (void *)bin);
+			set_value (stream->proptag, bin);
 		else if (stream->proptag == PR_BODY_UNICODE)
-			set_SPropValue_proptag(&props[i++], stream->proptag, (void *)stream->value->data);
+			set_value (stream->proptag, stream->value->data);
 	}
 
 	/*  FIXME : */
 	/* editor = EDITOR_FORMAT_PLAINTEXT; */
-	/* set_SPropValue_proptag(&props[i++], PR_MSG_EDITOR_FORMAT, (const void *)editor); */
+	/* set_value (PR_MSG_EDITOR_FORMAT, &editor); */
 
-	*value = props;
-	return i;
+	return TRUE;
 }
 
 /* -- Generate MIME to ITEM -- */
